<?php
namespace nur\sery\wip\schema\types;

use nulib\ValueException;
use nur\sery\wip\schema\input\Input;
use nur\sery\wip\schema\Result;
use nur\sery\wip\schema\Schema;

/**
 * Interface IType: un type de données
 */
interface IType {
  /** la donnée $input($destKey) est-elle disponible? */
  function isAvailable(Input $input, $destKey): bool;

  /** la valeur $value est-elle nulle? */
  function isNull($value): bool;

  /**
   * la valeur $value est-elle valide et normalisée le cas échéant?
   *
   * NB: si $value est un string. elle doit avoir déjà été traitée au préalable
   * par extract() et parse()
   */
  function isValid($value, ?bool &$normalized=null): bool;

  /** extraire de la chaine la valeur à analyser */
  function extract(string &$value): bool;

  /**
   * analyser la chaine et retourner la valeur "convertie"
   *
   * @throws ValueException en cas d'erreur d'analyse
   */
  function parse(string $value);

  /**
   * analyser, corriger éventuellement et normaliser la valeur
   *
   * NB: si $value est un string. elle doit avoir déjà été traitée au préalable
   * par extract() et parse()
   *
   * si la valeur était déjà normalisée, ou si une erreur s'est produite,
   * retourner false.
   */
  function verifix(&$value, Result $result, Schema $schema): bool;

  /**
   * formatter la valeur pour affichage. si $value n'est pas null, elle est
   * garantie d'être du bon type
   */
  function format($value, $format=null): string;
}
